package com.wjholden.nmap;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import android.content.Context;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.AsyncTask;
import android.os.Message;

/**
 * Perform the actual scan.
 * 
 * 0.4.5:
 * - moved many local variables to private visibility
 * - utilized previously unused NmapError class.
 * 
 * @author William John Holden (wjholden@gmail.com)
 * @version 0.4.5
 */
// Object[0]=String, Object[1]=Context
// Turns out the Context isn't even needed.
public class NmapScan extends AsyncTask <Object, Void, Void> implements NmapConstants
{
	private transient String command;
	
	private transient DataOutputStream os;
	private transient BufferedReader in;
	private transient BufferedReader err;
	private transient Process p;

	protected void onPreExecute()
	{
		switch (NmapMain.scanType) {
			case SCANTYPE_NMAP:
				command = "nmap ";
				break;
			case SCANTYPE_NPING:
				command = "nping ";
				break;
			case SCANTYPE_NDIFF:
				command = "ndiff ";
				break;
			case SCANTYPE_NCAT:
				command = "ncat ";
				break;
			default:
				command = "nmap ";
				break;
		}
		
		if (NmapMain.canGetRoot)
		{
			NmapMain.outputArgs += " --privileged ";
		}
		else
		{
			NmapMain.outputArgs += " --unprivileged ";
		}
		
		command += NmapMain.outputArgs;

		NmapMain.ne.log("Selected scan type: " + command + " (" + NmapMain.scanType + ")");
	}

	protected void onPostExecute(final Void myVoid)
	{
		if (NmapMain.pd != null)
		{
			NmapMain.pd.dismiss();
		}
		NmapMain.hasRunOneScan = true;
	}

	@Override
	protected Void doInBackground(final Object... params) {
		final Context context = (Context) params[0];
		final String exeParams = (String) params[1];

		if (NmapMain.canGetRoot)
		{
			NmapMain.ne.log("Getting root...");
		}
		
		try {
			final String temporaryFolder = NmapUtilities.verifyTemporaryFolder(context);
			if (NmapMain.scanType == SCANTYPE_NMAP)
			{
				NmapMain.outputArgs = " -oA " + temporaryFolder + "/nmap ";
			}
		} catch (NameNotFoundException e1) {
			NmapMain.ne.log(e1.getMessage());
		} catch (IOException e1) {
			NmapMain.ne.log(e1.getMessage());
		}
		
		try {
			p = Runtime.getRuntime().exec(NmapMain.sh);
		} catch (IOException e) {
			NmapMain.ne.log(e.getMessage());
			final Message msg = Message.obtain();
			msg.obj = "Unable to start shell: " + e.toString();
			msg.what = RUN_ERROR;
			NmapMain.handler.sendMessage(msg);
			return null;
		}
		if (NmapMain.canGetRoot && p != null)
		{
			NmapMain.ne.log("Got root!");
		}
		else if (p == null)
		{
			NmapMain.ne.log("Process p is null - did getRuntime().exec(Nmap.sh) fail?");
			final Message msg = Message.obtain();
			msg.obj = "Process for executing scan is null. Please close this program and reopen it.";
			msg.what = RUN_ERROR;
			NmapMain.handler.sendMessage(msg);
			return null;
		}
		
		os = new DataOutputStream(p.getOutputStream());
		in = new BufferedReader(new InputStreamReader(p.getInputStream()));
		err = new BufferedReader(new InputStreamReader(p.getErrorStream()));

		if (os == null)
		{
			NmapMain.ne.log("DataOutputStream 'os' is null!");
		}
		if (in == null)
		{
			NmapMain.ne.log("BufferedReader 'in' is null!");
		}
		if (err == null)
		{
			NmapMain.ne.log("BufferedReader 'err' is null!");
		}

		try {
			os.writeBytes("cd " + NmapMain.bindir + "\n");
			NmapMain.ne.log("cd " + NmapMain.bindir);
			os.flush();
			/* wtf? -h is not a special case, and you're not doing anything that different.
			 * Also, current code branching includes useless if statement.
			 */

			if (exeParams.equals("-h"))
				os.writeBytes("./" + command + " -h\n");
			else
				if (NmapMain.canGetRoot)
				{
					os.writeBytes("./" + command + exeParams + "\n");
					NmapMain.ne.log("./" + command + exeParams);
				}
				else
				{
					os.writeBytes("./" + command + exeParams + "\n");
					NmapMain.ne.log("./" + command + exeParams);
				}

			/** @since 21 Rewrite for shorter passage that makes common fucking sense */
			/* BEGIN @since 21 */
			/*{
				if (executionParameters == null)
					executionParameters = "";
				String myCompleteCommand = "./" + command + executionParameters + "\n"; 
				os.writeBytes(myCompleteCommand);
				Nmap.ne.Log(myCompleteCommand);
			}*/
			/* END @since 21 */
			os.flush();
			os.writeBytes("exit\n");
			NmapMain.ne.log("exit");
			os.flush();
			os.close();

			// Catch the errors first: 
			String error, errLine;
			error = errLine = "";
			while ((errLine = err.readLine()) != null)
			{
				NmapMain.ne.log(errLine);
				error += errLine;
			}
			if (error.length() > 0)
			{
				Message msg1 = Message.obtain();
				msg1.obj = "Error detected at runtime: " + error;
				msg1.what = RUN_ERROR;
				NmapMain.handler.sendMessage(msg1);
			}
		} catch (IOException e) {
			NmapMain.ne.log(e.getMessage());
			Message msg = Message.obtain();
			msg.obj = "IOException: " + e.toString();
			msg.what = RUN_ERROR;
			NmapMain.handler.sendMessage(msg);
		}

		String line;
		try {
			while ((line = in.readLine()) != null)
			{
				Message msg = Message.obtain();
				msg.obj = line;
				msg.what = RUN_LINE;
				NmapMain.handler.sendMessage(msg);
			}
		} catch (IOException e) {
			Message msg = Message.obtain();
			msg.obj = "Unable to read command output: " + e.toString();
			msg.what = RUN_ERROR;
			NmapMain.handler.sendMessage(msg);
		}
		try {
			in.close();
			err.close();
		} catch (IOException e) {
			Message msg = Message.obtain();
			msg.obj = "Unable to close command output: " + e.toString();
			msg.what = RUN_ERROR;
			NmapMain.handler.sendMessage(msg);
		}

		p.destroy();

		if (!exeParams.equals("-h"))
		{
			NmapMain.handler.sendEmptyMessage(RUN_COMPLETE);
		}
		return null;
	}
}
